﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;
using System.Drawing.Imaging;

namespace 撕纸效果
{
    public partial class MainForm : Form
    {
        #region 字段

        /// <summary>
        /// 撕纸方向
        /// </summary>
        private TearHelper.TearOrientation tearOrientation = TearHelper.TearOrientation.Bottom | TearHelper.TearOrientation.Right;
        /// <summary>
        /// 撕纸阴影方向
        /// </summary>
        private TearHelper.ShadowOrientation shadowOrientation = TearHelper.ShadowOrientation.Bottom | TearHelper.ShadowOrientation.Right;
        /// <summary>
        /// 图片路径集合
        /// </summary>
        private List<string> filepathList = new List<string>();
        /// <summary>
        /// 图片路径集合临时
        /// </summary>
        private List<string> filepathList_tmp = new List<string>();
        /// <summary>
        /// 原图
        /// </summary>
        private Image image_src = null;

        #endregion

        public MainForm()
        {
            InitializeComponent();

            this.preview_pb.AllowDrop = true;
        }

        #region 撕纸参数

        /// <summary>
        /// 撕纸参数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void image_param_nud_ValueChanged(object sender, EventArgs e)
        {
            this.UpdatePictureBox();
        }

        /// <summary>
        /// 当前图片
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void image_current_nud_ValueChanged(object sender, EventArgs e)
        {
            this.LoadImage(this.filepathList[(int)this.image_current_nud.Value - 1]);
        }

        #endregion

        #region 撕纸方向
        private void tear_top_cb_CheckedChanged(object sender, EventArgs e)
        {
            this.tearOrientation = this.tearOrientation ^ TearHelper.TearOrientation.Top;
            this.UpdatePictureBox();
        }
        private void tear_right_cb_CheckedChanged(object sender, EventArgs e)
        {
            this.tearOrientation = this.tearOrientation ^ TearHelper.TearOrientation.Right;
            this.UpdatePictureBox();
        }
        private void tear_bottom_cb_CheckedChanged(object sender, EventArgs e)
        {
            this.tearOrientation = this.tearOrientation ^ TearHelper.TearOrientation.Bottom;
            this.UpdatePictureBox();
        }
        private void tear_left_cb_CheckedChanged(object sender, EventArgs e)
        {
            this.tearOrientation = this.tearOrientation ^ TearHelper.TearOrientation.Left;
            this.UpdatePictureBox();
        }

        #endregion

        #region 撕纸阴影方向
        private void shadow_top_cb_CheckedChanged(object sender, EventArgs e)
        {
            this.shadowOrientation = this.shadowOrientation ^ TearHelper.ShadowOrientation.Top;
            this.UpdatePictureBox();
        }
        private void shadow_right_cb_CheckedChanged(object sender, EventArgs e)
        {
            this.shadowOrientation = this.shadowOrientation ^ TearHelper.ShadowOrientation.Right;
            this.UpdatePictureBox();
        }
        private void shadow_bottom_cb_CheckedChanged(object sender, EventArgs e)
        {
            this.shadowOrientation = this.shadowOrientation ^ TearHelper.ShadowOrientation.Bottom;
            this.UpdatePictureBox();
        }
        private void shadow_left_cb_CheckedChanged(object sender, EventArgs e)
        {
            this.shadowOrientation = this.shadowOrientation ^ TearHelper.ShadowOrientation.Left;
            this.UpdatePictureBox();
        }

        #endregion

        /// <summary>
        /// 添加图片按钮
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void open_btn_Click(object sender, EventArgs e)
        {
            try
            {
                this.open_ofd.Filter = "png;jpg;jpeg;bmp|*.png*;*.jpg;*.jpeg;*.bmp";
                this.open_ofd.RestoreDirectory = true;
                this.open_ofd.Title = "请选择图片";

                this.open_ofd.Multiselect = true;
                this.open_ofd.CheckFileExists = true;
                this.open_ofd.CheckPathExists = true;
                this.open_ofd.AddExtension = true;
                this.open_ofd.ReadOnlyChecked = false;

                if (this.open_ofd.ShowDialog() == DialogResult.OK)
                {
                    this.InitImages(this.open_ofd.FileNames);
                    this.LoadImage(this.filepathList[(int)this.image_current_nud.Value - 1]);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.StackTrace, "错误");
            }
        }

        /// <summary>
        /// 保存撕纸效果按钮
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void save_btn_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.image_src == null)
                    return;

                this.save_sfd.RestoreDirectory = true;
                this.save_sfd.Title = "请选择保存路径";
                this.save_sfd.FileName = "自动批量";
                this.save_sfd.CheckPathExists = true;
                this.save_sfd.OverwritePrompt = false;

                if (this.save_sfd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    for (int i = 0; i < this.filepathList.Count; i++)
                    {
                        string filename = Path.GetDirectoryName(this.save_sfd.FileName) + "\\撕纸效果_" + Path.GetFileNameWithoutExtension(this.filepathList[i]) + ".png";
                        if (File.Exists(filename) && this.coverage_cb.Checked == false)
                        {
                            continue;
                        }
                        Image image = (Image)TearHelper.TearIamge(this.image_src, this.tearOrientation, this.shadowOrientation, (int)this.tear_thickness_nud.Value, (int)this.tear_shadow_nud.Value, (int)this.tear_space_nud.Value);
                        image.Save(filename, ImageFormat.Png);
                        image.Dispose();
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.StackTrace, "错误");
            }
        }

        #region PictureBox
        private void preview_pb_DoubleClick(object sender, EventArgs e)
        {
            this.open_btn_Click(this, new EventArgs());
        }
        private void preview_pb_DragEnter(object sender, DragEventArgs e)
        {
            try
            {
                this.filepathList_tmp.Clear();
                e.Effect = DragDropEffects.Copy;
                string[] filepathArr = ((String[])e.Data.GetData(DataFormats.FileDrop, true));//必须用字符串数组
                for (int i = 0; i < filepathArr.Length; i++)
                {
                    if (System.Text.RegularExpressions.Regex.IsMatch(Path.GetExtension(filepathArr[i]), "(.png)|(.jpg)|(.jpeg)|(.bmp)", RegexOptions.IgnoreCase))
                    {
                        this.filepathList_tmp.Add(filepathArr[i]);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.StackTrace, "错误");
            }
        }
        private void preview_pb_DragLeave(object sender, EventArgs e)
        {
            try
            {
                this.filepathList_tmp.Clear();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.StackTrace, "错误");
            }
        }
        private void preview_pb_DragDrop(object sender, DragEventArgs e)
        {
            try
            {
                if (this.filepathList_tmp.Count > 0)
                {
                    this.filepathList = this.filepathList_tmp;
                    this.filepathList_tmp = new List<string>();
                    this.InitImages(this.filepathList.ToArray());
                    this.LoadImage(this.filepathList[(int)this.image_current_nud.Value - 1]);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.StackTrace, "错误");
            }
        }
        private void preview_pb_Paint(object sender, PaintEventArgs e)
        {
            if (this.image_src == null)
            {
                string text = "（双击、拖载）加载图片";
                SizeF text_size = e.Graphics.MeasureString(text, this.Font);
                SolidBrush text_sb = new SolidBrush(Color.Tomato);
                StringFormat text_sf = new StringFormat() { Alignment = StringAlignment.Center, LineAlignment = StringAlignment.Center };
                e.Graphics.DrawString(text, this.Font, text_sb, this.preview_pb.ClientRectangle, text_sf);
                text_sb.Dispose();
                text_sf.Dispose();
            }
        }

        #endregion

        #region 私有方法

        /// <summary>
        /// 加载所有图片地址
        /// </summary>
        /// <param name="filepaths"></param>
        private void InitImages(string[] filepaths)
        {
            this.filepathList.Clear();
            this.image_current_nud.ValueChanged -= this.image_current_nud_ValueChanged;
            this.image_current_nud.Minimum = 1;
            this.image_current_nud.Maximum = filepaths.Length;
            this.image_current_nud.Value = 1;
            this.image_current_nud.ValueChanged += this.image_current_nud_ValueChanged;
            image_sum_va.Text = filepaths.Length.ToString();
            for (int i = 0; i < filepaths.Length; i++)
            {
                this.filepathList.Add(filepaths[i]);
            }
        }

        /// <summary>
        /// 读取读取指定图片
        /// </summary>
        /// <param name="filepath">图片路径</param>
        private void LoadImage(string filepath)
        {
            if (!File.Exists(filepath))
                return;

            if (this.image_src != null)
                this.image_src.Dispose();

            Image bmp = Image.FromFile(filepath);
            this.image_src = bmp.GetThumbnailImage(bmp.Width, bmp.Height, null, IntPtr.Zero);
            bmp.Dispose();
            this.UpdatePictureBox();
        }

        /// <summary>
        /// 更新PictureBox界面
        /// </summary>
        private void UpdatePictureBox()
        {
            if (this.image_src != null)
            {
                this.preview_pb.BackgroundImage = (Image)TearHelper.TearIamge(this.image_src, this.tearOrientation, this.shadowOrientation, (int)this.tear_thickness_nud.Value, (int)this.tear_shadow_nud.Value, (int)this.tear_space_nud.Value);
            }
        }

        #endregion

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                if (this.image_src != null)
                {
                    this.image_src.Dispose();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.StackTrace, "错误");
            }
        }

    }
}
